# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"plot_images": "00_visualize.ipynb",
         "imshow": "00_visualize.ipynb",
         "show": "00_visualize.ipynb",
         "tensor2imgs": "00_visualize.ipynb",
         "bbox_visualize": "00_visualize.ipynb",
         "multi_thread": "01_process.ipynb",
         "identify": "03_memoize.ipynb",
         "memoize": "03_memoize.ipynb",
         "imemoize": "03_memoize.ipynb",
         "ICACHE": "03_memoize.ipynb",
         "VERBOSE": "03_memoize.ipynb",
         "CACHE_DIR": "03_memoize.ipynb",
         "is_interactive": "03_utils.ipynb",
         "get_name": "03_utils.ipynb",
         "get_files": "03_utils.ipynb",
         "find_contours": "03_utils.ipynb",
         "download_file_from_google_drive": "03_utils.ipynb",
         "mkdir": "03_utils.ipynb",
         "put_text": "03_utils.ipynb",
         "video_to_images": "03_utils.ipynb",
         "v2i": "03_utils.ipynb",
         "images_to_video": "03_utils.ipynb",
         "av_i2v": "03_utils.ipynb",
         "TimeLoger": "03_utils.ipynb",
         "generate_tmp_filename": "03_utils.ipynb",
         "get_md5": "03_utils.ipynb",
         "printc": "03_utils.ipynb",
         "make_mini_coco": "04_debug.ipynb",
         "dpython": "04_debug.ipynb",
         "AvCOCO": "05_coco_dataset.ipynb",
         "PYTHON_VERSION": "05_coco_dataset.ipynb",
         "CocoDataset": "05_coco_dataset.ipynb",
         "get_bboxes": "05_coco_dataset.ipynb",
         "get_overlap_rate": "05_coco_dataset.ipynb",
         "bbox_expand": "05_coco_dataset.ipynb",
         "DiagnoseCoco": "05_coco_dataset.ipynb",
         "to_jpg": "05_coco_dataset.ipynb",
         "video_to_coco": "05_coco_dataset.ipynb",
         "split_coco": "05_coco_dataset.ipynb",
         "concat_coco": "05_coco_dataset.ipynb",
         "check_save_coco_dict": "05_coco_dataset.ipynb",
         "extract_coco": "05_coco_dataset.ipynb",
         "v2c": "05_coco_dataset.ipynb",
         "convert_image": "06_cli.ipynb",
         "setup_distributed": "07_dist_utils.ipynb",
         "get_rank": "07_dist_utils.ipynb",
         "primary": "07_dist_utils.ipynb",
         "synchronize": "07_dist_utils.ipynb",
         "get_world_size": "07_dist_utils.ipynb",
         "reduce_sum": "07_dist_utils.ipynb",
         "gather_grad": "07_dist_utils.ipynb",
         "all_gather": "07_dist_utils.ipynb",
         "all_gatherv": "07_dist_utils.ipynb",
         "all_reduce": "07_dist_utils.ipynb",
         "rank0_to_all": "07_dist_utils.ipynb",
         "reduce_loss_dict": "07_dist_utils.ipynb",
         "__all__": "all.ipynb"}

modules = ["visualize.py",
           "process.py",
           "utils.py",
           "debug.py",
           "coco.py",
           "cli.py",
           "dist_utils.py",
           "all.py"]

doc_url = "https://anhvth.github.io/avcv/"

git_url = "https://github.com/anhvth/avcv/tree/nbdev-convert/"

def custom_doc_links(name): return None
